OBJ

  Pst : "Parallax Serial Terminal"
  F : "F32"
    
CON

  _CLKMODE = XTAL1 + PLL16X 
  _XINFREQ = 5_000_000

  
  I2C_CLOCK = 28       '**should this infomation be in the header? 
  I2C_DATA = 29

 
  WRITE_DATA     = $3C ' Requests Write operation 
  READ_DATA      = $3D ' Requests Read operation
  MODE           = $02 ' Mode setting register
  OUTPUT_X_MSB   = $03 ' X MSB data output register  

VAR

  long x
  long y
  long z
  long topX, topY, topZ, pseudoHeading, pseudoTiltX, pseudoTiltY
  long compassPtr
         
PUB Main

  compassPtr := @topX
  F.Start
  waitcnt(clkfreq/100_000 + cnt)     ' Power up delay
  pst.start(115200)
  SetCont
  repeat
    SetPointer(OUTPUT_X_MSB)
    getRaw                           ' Gather raw data from compass       
    pst.Char(1)
    ShowVals

PUB SetCont
  ' Sets compass to continuous output mode
 
  start
  send(WRITE_DATA)
  send(MODE)
  send($00)
  stop

PUB SetPointer(Register)
  ' Start pointer at user specified register (OUT_X_MSB)

  start
  send(WRITE_DATA)
  send(Register)
  stop

PUB GetRaw | fX, fY, fZ 
  ' Get raw data from continous output

  start
  send(READ_DATA)
  x := ((receive(true) << 8) | receive(true))
  z := ((receive(true) << 8) | receive(true))
  y := ((receive(true) << 8) | receive(false))
  stop
  ~~x
  ~~z
  ~~y
 { x := x       
  z := z 
  y := y            }
  long[compassPtr] := x
  long[compassPtr][1] := y
  long[compassPtr][2] := z
  fX := F.FFloat(x)
  fY := F.FFloat(y)
  fZ := F.FFloat(z)
  
  long[compassPtr][3] := F.FRound(F.FMul(F.Degrees(F.ATan2(fY, fX)), 1000.0))
  long[compassPtr][4] := F.FRound(F.FMul(F.Degrees(F.ATan2(fZ, fX)), 1000.0))
  long[compassPtr][5] := F.FRound(F.FMul(F.Degrees(F.ATan2(fZ, fY)), 1000.0))
  
PUB ShowVals
  ' Display XYZ compass values
        
  Pst.Position(0, 0)
  Pst.str(string("X = "))        
  Pst.dec(x)
  Pst.str(string("   "))
  Pst.PositionX(8) 
  Pst.str(string(", Y = "))
  Pst.dec(y)
  Pst.str(string("   "))
  Pst.PositionX(18) 
  pst.str(string(", Z ="))
  pst.dec(z)
  pst.ClearEnd
  Pst.Position(0, 2)
  pst.str(string("Course = "))
  DecPoint(long[compassPtr][3], 1000)       
  'pst.dec(long[compassPtr][3])
  pst.str(string("   "))
  Pst.PositionX(17) 
  pst.str(string(", tiltX = "))
  DecPoint(long[compassPtr][4], 1000)       
  'pst.dec(long[compassPtr][4])
  pst.str(string("   "))
  Pst.PositionX(33) 
  pst.str(string(", tiltY = "))
  DecPoint(long[compassPtr][5], 1000)       
  'pst.dec(long[compassPtr][5])
  Pst.ClearEnd
  

PRI send(value)

  value := ((!value) >< 8)

  repeat 8
    dira[I2C_DATA]  := value
    dira[I2C_CLOCK] := false
    dira[I2C_CLOCK] := true
    value >>= 1

  dira[I2C_DATA]  := false
  dira[I2C_CLOCK] := false
  result         := !(ina[I2C_DATA])
  dira[I2C_CLOCK] := true
  dira[I2C_DATA]  := true

PRI receive(aknowledge)

  dira[I2C_DATA] := false

  repeat 8
    result <<= 1
    dira[I2C_CLOCK] := false
    result         |= ina[I2C_DATA]
    dira[I2C_CLOCK] := true

  dira[I2C_DATA]  := aknowledge
  dira[I2C_CLOCK] := false
  dira[I2C_CLOCK] := true
  dira[I2C_DATA]  := true

PRI start

  outa[I2C_DATA]  := false
  outa[I2C_CLOCK] := false
  dira[I2C_DATA]  := true
  dira[I2C_CLOCK] := true

PRI stop

  dira[I2C_CLOCK] := false
  dira[I2C_DATA]  := false

PUB DecPoint(value, denominator)
  if value < 0
    Pst.Char("-")
    -value
      
  if value => denominator
    result := value / denominator
    Pst.Dec(result)
    value //= denominator     
  else    
    Pst.Char("0")
  Pst.Char(".")  
  repeat while denominator > 1
    denominator /= 10
    if value => denominator
      result := value / denominator
      Pst.Dec(result)
      value //= denominator
    else
      Pst.Char("0")
  